"""Tests for tokenizer_mecab."""
from unittest import TestCase
from unittest.mock import patch, call
import tempfile
import os.path
import MeCab
import joblib
import serialmecab.tokenizer as t


class TestMecabTokenizer(TestCase):
    """Tests for MecabTokenizer."""

    def setUp(self):
        self.tokenizer = t.MecabTokenizer(MeCab.Tagger())
        self.tempfile = tempfile.mktemp('mecab_tokenizer')

    def tearDown(self):
        if os.path.exists(self.tempfile):
            os.remove(self.tempfile)

    def test_instance(self):
        """MecabTokenizer extends Tokenizer class."""
        self.assertIsInstance(
            self.tokenizer, t.MecabTokenizer)

    def test_mecab(self):
        """MecabTokenizer uses MeCab for morphological analysis."""
        self.assertIsInstance(
            self.tokenizer.tagger, MeCab.Tagger)

    def test_callable(self):
        """Tokenize a text when it is called as a function."""
        tokens = self.tokenizer(
            '資本主義における覚悟は、破産と失業である')
        self.assertIsInstance(tokens, list)
        for token in tokens:
            self.assertIsInstance(token, str)

    def test_persistent(self):
        """MecabTokenizer is picklable."""
        joblib.dump(self.tokenizer, self.tempfile)

    def test_unpicklable(self):
        """A pickled MecabTokenizer is deserializable."""
        joblib.dump(self.tokenizer, self.tempfile)
        tokenizer = joblib.load(self.tempfile)

        self.assertIsInstance(tokenizer.tagger, MeCab.Tagger)

    def test_create_tagger(self):
        """Create a `MeCab.Tagger` object."""
        self.assertIsInstance(self.tokenizer._create_tagger(),
                              MeCab.Tagger)

    @patch('serialmecab.tokenizer.MecabTokenizer._create_tagger')
    def test_no_arg(self, create_tagger):
        """An argument is optional."""
        tokenizer = self.tokenizer.create()
        self.assertIsInstance(tokenizer, t.MecabTokenizer)
        self.assertEqual(create_tagger.call_args_list, [call()])

    @patch('MeCab.Tagger')
    def test_use_mecab_dicdir(self, tagger):
        """Accept mecab dicdir, and pass it to Mecab.Tagger."""
        pseudo_dicdir = 'path/to/dic/dir'

        self.tokenizer._create_tagger(pseudo_dicdir)

        self.assertEqual(tagger.call_args_list,
                         [call(f'-d {pseudo_dicdir}')])

    @patch('MeCab.Tagger')
    def test_not_use_mecab_dicdir(self, tagger):
        """dicdir is optional."""

        self.tokenizer._create_tagger()

        self.assertEqual(tagger.call_args_list, [call()])



"""Expose a class for tokenization."""
from logging import getLogger
from typing import List
import os
import MeCab


class MecabTokenizer:
    """A Tokenizer that uses MeCab."""

    logger = getLogger(__name__)

    def __init__(self, tagger: MeCab.Tagger):
        """Take a client for MeCab."""
        self.tagger = tagger

    def __call__(self, text: str) -> List[str]:
        """Tokenize a text."""
        segments = self.tagger.parse(text)
        tokens = [segment.split('\t')[0]
                  for segment in segments.split(os.linesep)][:-2]
        return tokens

    def __getstate__(self):
        """Exclude :py:attr:`tagger` because it is not picklable."""
        return {}

    def __setstate__(self, _):
        """Revert :py:attr:`tagger`.

        Set :py:attr:`tagger` to a new :py:class:`MeCab.Tagger` object.
        """
        self.tagger = self._create_tagger()
        self.logger = getLogger(__name__)

    @classmethod
    def create(cls):
        """Create a :py:class:`MecabTokenizer` object."""
        return MecabTokenizer(cls._create_tagger())

    @classmethod
    def _create_tagger(
            cls, dicdir=os.getenv('MECAB_DICDIR', None)) -> MeCab.Tagger:
        """Create a :py:class:`MeCab.Tagger` object."""
        cls.logger.info('mecab system dictionary is %s.', dicdir)
        return MeCab.Tagger(f'-d {dicdir}') if dicdir else MeCab.Tagger()



"""Metadata of the package."""
from setuptools import setup, find_packages


setup(
    name='serial-mecab',
    version="1.0.0",
    description=('mecab-python3 wrapper for serialization'),
    packages=find_packages(),
    install_requires=[
        'mecab-python3',
        'joblib'
    ],
    extras_require={
        'test': [
            'pytest'
        ],
        'dev': [
            'ipython',
            'python-language-server[all]'
        ],
        'doc': [
            'sphinx',
            'sphinx_rtd_theme'
        ]})
